@@ -39,6 +39,20 @@ LWTIN* lwtin_from_geos(const GEOSGeometry* geom, uint8_t want3d);
 #define LWGEOM_GEOS_ERRMSG_MAXSIZE 256
 char lwgeom_geos_errmsg[LWGEOM_GEOS_ERRMSG_MAXSIZE];
 
+const char *
+lwgeom_geos_compiled_version()
+{
+	static char ver[64];
+	sprintf(
+		ver,
+		"%d.%d.%d",
+		(POSTGIS_GEOS_VERSION/10000),
+		((POSTGIS_GEOS_VERSION%10000)/100),
+		((POSTGIS_GEOS_VERSION)%100)
+	);
+	return ver;
+}
+
 extern void
 lwgeom_geos_error(const char* fmt, ...)
 {
@@ -52,6 +66,16 @@ lwgeom_geos_error(const char* fmt, ...)
 	va_end(ap);
 }
 
+void
+lwgeom_geos_error_minversion(const char *functionality, const char *minver)
+{
+	lwerror(
+		"%s requires a build against GEOS-%s or higher,"
+		" this version of PostGIS was built against version %s",
+		functionality, minver, lwgeom_geos_compiled_version()
+	);
+}
+
 /* Destroy any non-null GEOSGeometry* pointers passed as arguments */
 #define GEOS_FREE(...) \
 do { \
@@ -283,16 +307,14 @@ ptarray_to_GEOSCoordSeq(const POINTARRAY* pa, uint8_t fix_ring)
 		sq = GEOSCoordSeq_copyFromBuffer((const double*) pa->serialized_pointlist, pa->npoints, FLAGS_GET_Z(pa->flags), FLAGS_GET_M(pa->flags));
 		if (!sq)
 		{
-			lwerror("Error creating GEOS Coordinate Sequence");
-			return NULL;
+			GEOS_FAIL();
 		}
 		return sq;
 	}
 #endif
-        if (!(sq = GEOSCoordSeq_create(pa->npoints + append_points, dims)))
+	if (!(sq = GEOSCoordSeq_create(pa->npoints + append_points, dims)))
 	{
-		lwerror("Error creating GEOS Coordinate Sequence");
-		return NULL;
+		GEOS_FAIL();
 	}
 
 	for (i = 0; i < pa->npoints; i++)
@@ -413,7 +435,7 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 	GEOSGeom g, shell;
 	GEOSGeom* geoms = NULL;
 	uint32_t ngeoms, i, j;
-	int geostype;
+	int is_empty = LW_FALSE;
 #if LWDEBUG_LEVEL >= 4
 	char* wkt;
 #endif
@@ -435,144 +457,156 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 		return g;
 	}
 
-	LWPOINT* lwp = NULL;
-	LWPOLY* lwpoly = NULL;
-	LWLINE* lwl = NULL;
-	LWCOLLECTION* lwc = NULL;
+	is_empty = lwgeom_is_empty(lwgeom);
 
 	switch (lwgeom->type)
 	{
-	case POINTTYPE:
-		lwp = (LWPOINT*)lwgeom;
-
-		if (lwgeom_is_empty(lwgeom))
-			g = GEOSGeom_createEmptyPoint();
-		else
+		case POINTTYPE:
 		{
-#if POSTGIS_GEOS_VERSION < 30800
-			sq = ptarray_to_GEOSCoordSeq(lwp->point, 0);
-			g = GEOSGeom_createPoint(sq);
-#else
-			if (lwgeom_has_z(lwgeom))
-			{
-				sq = ptarray_to_GEOSCoordSeq(lwp->point, 0);
-				g = GEOSGeom_createPoint(sq);
-			}
+			if (is_empty)
+				g = GEOSGeom_createEmptyPoint();
 			else
 			{
-				const POINT2D* p = getPoint2d_cp(lwp->point, 0);
-				g = GEOSGeom_createPointFromXY(p->x, p->y);
+				LWPOINT* lwp = (LWPOINT*)lwgeom;
+	#if POSTGIS_GEOS_VERSION < 30800
+				sq = ptarray_to_GEOSCoordSeq(lwp->point, 0);
+				g = GEOSGeom_createPoint(sq);
+	#else
+				if (lwgeom_has_z(lwgeom))
+				{
+					sq = ptarray_to_GEOSCoordSeq(lwp->point, 0);
+					g = GEOSGeom_createPoint(sq);
+				}
+				else
+				{
+					const POINT2D* p = getPoint2d_cp(lwp->point, 0);
+					g = GEOSGeom_createPointFromXY(p->x, p->y);
+				}
+	#endif
 			}
-#endif
+			if (!g) return NULL;
+			break;
 		}
-		if (!g) return NULL;
-		break;
 
-	case LINETYPE:
-		lwl = (LWLINE*)lwgeom;
-		/* TODO: if (autofix) */
-		if (lwl->points->npoints == 1)
+		case LINETYPE:
 		{
-			/* Duplicate point, to make geos-friendly */
-			lwl->points = ptarray_addPoint(lwl->points,
-						       getPoint_internal(lwl->points, 0),
-						       FLAGS_NDIMS(lwl->points->flags),
-						       lwl->points->npoints);
+			if (is_empty)
+				g = GEOSGeom_createEmptyLineString();
+			else
+			{
+				LWLINE* lwl = (LWLINE*)lwgeom;
+				/* TODO: if (autofix) */
+				if (lwl->points->npoints == 1)
+				{
+					/* Duplicate point, to make geos-friendly */
+					lwl->points = ptarray_addPoint(lwl->points,
+								       getPoint_internal(lwl->points, 0),
+								       FLAGS_NDIMS(lwl->points->flags),
+								       lwl->points->npoints);
+				}
+				sq = ptarray_to_GEOSCoordSeq(lwl->points, 0);
+				g = GEOSGeom_createLineString(sq);
+			}
+			if (!g) return NULL;
+			break;
 		}
-		sq = ptarray_to_GEOSCoordSeq(lwl->points, 0);
-		g = GEOSGeom_createLineString(sq);
-		if (!g) return NULL;
-		break;
 
-	case POLYGONTYPE:
-		lwpoly = (LWPOLY*)lwgeom;
-		if (lwgeom_is_empty(lwgeom))
-			g = GEOSGeom_createEmptyPolygon();
-		else
+		case POLYGONTYPE:
 		{
-			shell = ptarray_to_GEOSLinearRing(lwpoly->rings[0], autofix);
-			if (!shell) return NULL;
-			ngeoms = lwpoly->nrings - 1;
+			LWPOLY* lwpoly = (LWPOLY*)lwgeom;
+			if (is_empty)
+				g = GEOSGeom_createEmptyPolygon();
+			else
+			{
+				shell = ptarray_to_GEOSLinearRing(lwpoly->rings[0], autofix);
+				if (!shell) return NULL;
+				ngeoms = lwpoly->nrings - 1;
+				if (ngeoms > 0) geoms = lwalloc(sizeof(GEOSGeom) * ngeoms);
+
+				for (i = 1; i < lwpoly->nrings; i++)
+				{
+					geoms[i - 1] = ptarray_to_GEOSLinearRing(lwpoly->rings[i], autofix);
+					if (!geoms[i - 1])
+					{
+						uint32_t k;
+						for (k = 0; k < i - 1; k++)
+							GEOSGeom_destroy(geoms[k]);
+						lwfree(geoms);
+						GEOSGeom_destroy(shell);
+						return NULL;
+					}
+				}
+				g = GEOSGeom_createPolygon(shell, geoms, ngeoms);
+				if (geoms) lwfree(geoms);
+			}
+			if (!g) return NULL;
+			break;
+		}
+
+		case TRIANGLETYPE:
+		{
+			if (is_empty)
+				g = GEOSGeom_createEmptyPolygon();
+			else
+			{
+				LWTRIANGLE *lwt = (LWTRIANGLE *)lwgeom;
+				shell = ptarray_to_GEOSLinearRing(lwt->points, autofix);
+				if (!shell) return NULL;
+				g = GEOSGeom_createPolygon(shell, NULL, 0);
+			}
+			if (!g) return NULL;
+			break;
+		}
+		case MULTIPOINTTYPE:
+		case MULTILINETYPE:
+		case MULTIPOLYGONTYPE:
+		case TINTYPE:
+		case COLLECTIONTYPE:
+		{
+			int geostype;
+			if (lwgeom->type == MULTIPOINTTYPE)
+				geostype = GEOS_MULTIPOINT;
+			else if (lwgeom->type == MULTILINETYPE)
+				geostype = GEOS_MULTILINESTRING;
+			else if (lwgeom->type == MULTIPOLYGONTYPE)
+				geostype = GEOS_MULTIPOLYGON;
+			else
+				geostype = GEOS_GEOMETRYCOLLECTION;
+
+			LWCOLLECTION* lwc = (LWCOLLECTION*)lwgeom;
+
+			ngeoms = lwc->ngeoms;
 			if (ngeoms > 0) geoms = lwalloc(sizeof(GEOSGeom) * ngeoms);
 
-			for (i = 1; i < lwpoly->nrings; i++)
+			j = 0;
+			for (i = 0; i < ngeoms; ++i)
 			{
-				geoms[i - 1] = ptarray_to_GEOSLinearRing(lwpoly->rings[i], autofix);
-				if (!geoms[i - 1])
+				GEOSGeometry* g;
+
+				/* if (lwgeom_is_empty(lwc->geoms[i])) continue; */
+
+				g = LWGEOM2GEOS(lwc->geoms[i], 0);
+				if (!g)
 				{
 					uint32_t k;
-					for (k = 0; k < i - 1; k++)
+					for (k = 0; k < j; k++)
 						GEOSGeom_destroy(geoms[k]);
 					lwfree(geoms);
-					GEOSGeom_destroy(shell);
 					return NULL;
 				}
+				geoms[j++] = g;
 			}
-			g = GEOSGeom_createPolygon(shell, geoms, ngeoms);
-			if (geoms) lwfree(geoms);
+			g = GEOSGeom_createCollection(geostype, geoms, j);
+			if (ngeoms > 0) lwfree(geoms);
+			if (!g) return NULL;
+			break;
 		}
-		if (!g) return NULL;
-		break;
 
-	case TRIANGLETYPE:
-		if (lwgeom_is_empty(lwgeom))
-			g = GEOSGeom_createEmptyPolygon();
-		else
+		default:
 		{
-			LWTRIANGLE *lwt = (LWTRIANGLE *)lwgeom;
-			shell = ptarray_to_GEOSLinearRing(lwt->points, autofix);
-			if (!shell)
-				return NULL;
-			g = GEOSGeom_createPolygon(shell, NULL, 0);
-		}
-		if (!g)
+			lwerror("Unknown geometry type: %d - %s", lwgeom->type, lwtype_name(lwgeom->type));
 			return NULL;
-		break;
-	case MULTIPOINTTYPE:
-	case MULTILINETYPE:
-	case MULTIPOLYGONTYPE:
-	case TINTYPE:
-	case COLLECTIONTYPE:
-		if (lwgeom->type == MULTIPOINTTYPE)
-			geostype = GEOS_MULTIPOINT;
-		else if (lwgeom->type == MULTILINETYPE)
-			geostype = GEOS_MULTILINESTRING;
-		else if (lwgeom->type == MULTIPOLYGONTYPE)
-			geostype = GEOS_MULTIPOLYGON;
-		else
-			geostype = GEOS_GEOMETRYCOLLECTION;
-
-		lwc = (LWCOLLECTION*)lwgeom;
-
-		ngeoms = lwc->ngeoms;
-		if (ngeoms > 0) geoms = lwalloc(sizeof(GEOSGeom) * ngeoms);
-
-		j = 0;
-		for (i = 0; i < ngeoms; ++i)
-		{
-			GEOSGeometry* g;
-
-			/* if (lwgeom_is_empty(lwc->geoms[i])) continue; */
-
-			g = LWGEOM2GEOS(lwc->geoms[i], 0);
-			if (!g)
-			{
-				uint32_t k;
-				for (k = 0; k < j; k++)
-					GEOSGeom_destroy(geoms[k]);
-				lwfree(geoms);
-				return NULL;
-			}
-			geoms[j++] = g;
 		}
-		g = GEOSGeom_createCollection(geostype, geoms, j);
-		if (ngeoms > 0) lwfree(geoms);
-		if (!g) return NULL;
-		break;
-
-	default:
-		lwerror("Unknown geometry type: %d - %s", lwgeom->type, lwtype_name(lwgeom->type));
-		return NULL;
 	}
 
 	GEOSSetSRID(g, lwgeom->srid);
@@ -580,7 +614,7 @@ LWGEOM2GEOS(const LWGEOM* lwgeom, uint8_t autofix)
 #if LWDEBUG_LEVEL >= 4
 	wkt = GEOSGeomToWKT(g);
 	LWDEBUGF(4, "LWGEOM2GEOS: GEOSGeom: %s", wkt);
-	free(wkt);
+	GEOSFree(wkt);
 #endif
 
 	return g;
@@ -726,7 +760,7 @@ lwgeom_intersection_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 
 	if ( prec >= 0) {
 #if POSTGIS_GEOS_VERSION < 30900
-		lwerror("Fixed-precision intersection requires GEOS-3.9 or higher");
+		lwgeom_geos_error_minversion("Fixed-precision intersection", "3.9");
 		GEOS_FREE_AND_FAIL(g1, g2);
 		return NULL;
 #else
@@ -774,7 +808,7 @@ lwgeom_linemerge_directed(const LWGEOM* geom, int directed)
 	if (directed)
 	{
 #if POSTGIS_GEOS_VERSION < 31100
-		lwerror("Directed line merging requires GEOS-3.11 or higher");
+		lwgeom_geos_error_minversion("Directed line merging", "3.11");
 		GEOS_FREE_AND_FAIL(g1);
 		return NULL;
 #else
@@ -823,7 +857,7 @@ lwgeom_unaryunion_prec(const LWGEOM* geom, double prec)
 
 	if ( prec >= 0) {
 #if POSTGIS_GEOS_VERSION < 30900
-		lwerror("Fixed-precision union requires GEOS-3.9 or higher");
+		lwgeom_geos_error_minversion("Fixed-precision unary union", "3.9");
 		GEOS_FREE_AND_FAIL(g1);
 		return NULL;
 #else
@@ -875,7 +909,7 @@ lwgeom_difference_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 
 	if ( prec >= 0) {
 #if POSTGIS_GEOS_VERSION < 30900
-		lwerror("Fixed-precision difference requires GEOS-3.9 or higher");
+		lwgeom_geos_error_minversion("Fixed-precision difference", "3.9");
 		GEOS_FREE_AND_FAIL(g1, g2);
 		return NULL;
 #else
@@ -926,7 +960,7 @@ lwgeom_symdifference_prec(const LWGEOM* geom1, const LWGEOM* geom2, double prec)
 
 	if ( prec >= 0) {
 #if POSTGIS_GEOS_VERSION < 30900
-		lwerror("Fixed-precision difference requires GEOS-3.9 or higher");
+		lwgeom_geos_error_minversion("Fixed-precision symdifference", "3.9");
 		GEOS_FREE_AND_FAIL(g1, g2);
 		return NULL;
 #else
@@ -985,7 +1019,7 @@ LWGEOM*
 lwgeom_reduceprecision(const LWGEOM* geom, double gridSize)
 {
 #if POSTGIS_GEOS_VERSION < 30900
-	lwerror("Precision reduction requires GEOS-3.9 or higher");
+	lwgeom_geos_error_minversion("Precision reduction", "3.9");
 	return NULL;
 #else
 
@@ -1079,7 +1113,7 @@ lwgeom_union_prec(const LWGEOM* geom1, const LWGEOM* geom2, double gridSize)
 
 	if ( gridSize >= 0) {
 #if POSTGIS_GEOS_VERSION < 30900
-		lwerror("Fixed-precision union requires GEOS-3.9 or higher");
+		lwgeom_geos_error_minversion("Fixed-precision union", "3.9");
 		GEOS_FREE_AND_FAIL(g1, g2);
 		return NULL;
 #else
@@ -1714,8 +1748,9 @@ lwpoly_to_points(const LWPOLY* lwpoly, uint32_t npoints, int32_t seed)
 		return NULL;
 	}
 
-	/* Gross up our test set a bit to increase odds of getting coverage in one pass */
-	sample_npoints = npoints * bbox_area / area;
+	/* Gross up our test set a bit (but not too much) to increase
+	 * odds of getting coverage in one pass */
+	sample_npoints = npoints * FP_MIN(bbox_area / area, 10000.0);
 
 	/* We're going to generate points using a sample grid as described
 	 * http://lin-ear-th-inking.blogspot.ca/2010/05/more-random-points-in-jts.html to try and get a more uniform
