@@ -56,16 +56,12 @@ bool gidx_is_unknown(const GIDX *a);
 /* Generate human readable form for GIDX. */
 char *gidx_to_string(GIDX *a);
 
-/* typedef to correct array-bounds checking for casts to GIDX - do not
-   use this ANYWHERE except in the casts below */
-typedef float _gidx_float_array[sizeof(float) * 2 * 4];
-
 /* Returns number of dimensions for this GIDX */
 #define GIDX_NDIMS(gidx) ((VARSIZE((gidx)) - VARHDRSZ) / (2 * sizeof(float)))
 /* Minimum accessor. */
-#define GIDX_GET_MIN(gidx, dimension) (*((_gidx_float_array *)(&(gidx)->c)))[2 * (dimension)]
+#define GIDX_GET_MIN(gidx, dimension) *( (gidx)->c + 2 * (dimension) )
 /* Maximum accessor. */
-#define GIDX_GET_MAX(gidx, dimension) (*((_gidx_float_array *)(&(gidx)->c)))[2 * (dimension) + 1]
+#define GIDX_GET_MAX(gidx, dimension) *( (gidx)->c + 2 * (dimension) + 1 )
 /* Minimum setter. */
 #define GIDX_SET_MIN(gidx, dimension, value) ((gidx)->c[2 * (dimension)] = (value))
 /* Maximum setter. */
@@ -100,8 +96,9 @@ int box2df_to_gbox_p(BOX2DF *a, GBOX *box);
 ** Fast functions for pulling boxes out of serializations.
 */
 
-/* Pull out the #GIDX bounding box with a absolute minimum system overhead */
+/* Pull out the #GIDX bounding box and flags with a absolute minimum system overhead */
 int gserialized_datum_get_gidx_p(Datum gserialized_datum, GIDX *gidx);
+int gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df);
 
 bool box2df_contains(const BOX2DF *a, const BOX2DF *b);
 void box2df_set_empty(BOX2DF *a);
@@ -123,4 +120,3 @@ void gidx_set_unknown(GIDX *a);
 bool gidx_overlaps(GIDX *a, GIDX *b);
 bool gidx_equals(GIDX *a, GIDX *b);
 bool gidx_contains(GIDX *a, GIDX *b);
-int gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df);
