@@ -68,6 +68,7 @@ usage()
 	printf(_( "  -X <tablespace> Specify the tablespace for the table's indexes.\n"
                   "      This applies to the primary key, and the spatial index if\n"
                   "      the -I flag is used.\n" ));
+	printf(_( "  -Z  Prevent tables from being analyzed.\n" ));
 	printf(_( "  -?  Display this help screen.\n" ));
 	printf( "\n" );
 	printf(_( "  An argument of `--' disables further option processing.\n" ));
@@ -85,9 +86,11 @@ main (int argc, char **argv)
 	int ret, i;
 
 #ifdef ENABLE_NLS
+#ifdef PGSQL_LOCALEDIR
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, PGSQL_LOCALEDIR);
 	textdomain (PACKAGE);
+#endif
 #endif
 
 	/* If no options are specified, display usage */
@@ -102,7 +105,7 @@ main (int argc, char **argv)
 	set_loader_config_defaults(config);
 
 	/* Keep the flag list alphabetic so it's easy to see what's left. */
-	while ((c = pgis_getopt(argc, argv, "-acdeg:ikm:nps:t:wDGIN:ST:W:X:")) != EOF)
+	while ((c = pgis_getopt(argc, argv, "-acdeg:ikm:nps:t:wDGIN:ST:W:X:Z")) != EOF)
 	{
 		// can not do this inside the switch case
 		if ('-' == c)
@@ -125,6 +128,10 @@ main (int argc, char **argv)
 			config->geography = 1;
 			break;
 
+		case 'Z':
+			config->analyze = 0;
+			break;
+
 		case 'S':
 			config->simple_geometries = 1;
 			break;
