@@ -27,14 +27,22 @@
 #include "lwgeom_cache.h"
 
 int geography_dwithin_cache(FunctionCallInfo fcinfo,
-			    const GSERIALIZED *g1,
-			    const GSERIALIZED *g2,
-			    const SPHEROID *s,
-			    double tolerance,
-			    int *dwithin);
+	SHARED_GSERIALIZED *g1,
+	SHARED_GSERIALIZED *g2,
+	const SPHEROID *s,
+	double tolerance,
+	int *dwithin);
 int geography_distance_cache(FunctionCallInfo fcinfo,
-			     const GSERIALIZED *g1,
-			     const GSERIALIZED *g2,
-			     const SPHEROID *s,
-			     double *distance);
-int geography_tree_distance(const GSERIALIZED* g1, const GSERIALIZED* g2, const SPHEROID* s, double tolerance, double* distance);
+	SHARED_GSERIALIZED *g1,
+	SHARED_GSERIALIZED *g2,
+	const SPHEROID *s,
+	double *distance);
+
+int geography_tree_distance(
+	const GSERIALIZED* g1,
+	const GSERIALIZED* g2,
+	const SPHEROID* s,
+	double tolerance,
+	double* distance);
+
+
