@@ -129,19 +129,27 @@
  *   datum is copied for use.
  *****************************************************************************/
 
-#include <postgres.h> /* for palloc */
-#include <fmgr.h> /* for PG_MODULE_MAGIC */
+/* PostgreSQL */
+#include "postgres.h" /* for palloc */
+#include "fmgr.h" /* for PG_MODULE_MAGIC */
+#include "libpq/pqsignal.h"
 #include "utils/guc.h"
+#include "utils/builtins.h"
 #include "utils/memutils.h"
+#include "utils/elog.h"
 
-#include "../../postgis_config.h"
+/* PostGIS */
+#include "postgis_config.h"
+#include "liblwgeom.h"
 #include "lwgeom_pg.h"
-
-#include "rtpostgis.h"
-#include "rtpg_internal.h"
 #include "stringlist.h"
 #include "optionlist.h"
 
+/* PostGIS Raster */
+#include "rtpostgis.h"
+#include "rtpg_internal.h"
+
+
 #ifndef __GNUC__
 # define __attribute__ (x)
 #endif
@@ -452,8 +460,8 @@ rt_pg_vsi_check_options(char **newval, void **extra, GucSource source)
 
 static char *gdal_datapath = NULL;
 static char *gdal_vsi_options = NULL;
-extern char *gdal_enabled_drivers;
-extern bool enable_outdb_rasters;
+static char *gdal_enabled_drivers = NULL;
+static bool enable_outdb_rasters = false;
 
 /* ---------------------------------------------------------------- */
 /*  Useful variables                                                */
@@ -638,6 +646,48 @@ rtpg_assignHookEnableOutDBRasters(bool enable, void *extra) {
 }
 
 
+
+/*
+* Machinery for intercepting the system SIGINT
+* handler so we can cancel long-running GDAL operations
+* via the progress handlers.
+*/
+static pqsigfunc coreIntHandler = 0;
+
+#ifdef WIN32
+static void interruptCallback() {
+  if (UNBLOCKED_SIGNAL_QUEUE())
+    pgwin32_dispatch_queued_signals();
+}
+#endif
+
+
+/*
+* This is the interrupt capture for this module.
+* Before handing off the signal to the core, it
+* sets the interrupt flag for currently running
+* functions.
+*/
+static void
+handleInterrupt(int sig)
+{
+	/*
+	* NOTE: printf here would be dangerous, see
+	* https://trac.osgeo.org/postgis/ticket/3644
+	*/
+	// printf("postgis_raster interrupt requested\n"); fflush(stdout);
+
+	/* Request interruption of liblwgeom as well */
+	lwgeom_request_interrupt();
+
+	/* Pass control into the usual core handler */
+	if (coreIntHandler) {
+		(*coreIntHandler)(sig);
+	}
+}
+
+
+
 /* Module load callback */
 void
 _PG_init(void) {
@@ -645,6 +695,15 @@ _PG_init(void) {
 	bool boot_postgis_enable_outdb_rasters = false;
 	MemoryContext old_context;
 
+	/* Set up interrupt capture */
+	coreIntHandler = pqsignal(SIGINT, handleInterrupt);
+
+#ifdef WIN32
+ 	GEOS_interruptRegisterCallback(interruptCallback);
+ 	lwgeom_register_interrupt_callback(interruptCallback);
+#endif
+
+
 	/*
 	 * Change to context for memory allocation calls like palloc() in the
 	 * extension initialization routine
@@ -804,9 +863,12 @@ _PG_init(void) {
 void
 _PG_fini(void) {
 
-	MemoryContext old_context;
+	MemoryContext old_context = MemoryContextSwitchTo(TopMemoryContext);
 
-	old_context = MemoryContextSwitchTo(TopMemoryContext);
+	elog(NOTICE, "Goodbye from PostGIS Raster %s", POSTGIS_VERSION);
+
+	/* Return SIGINT handling to core */
+	pqsignal(SIGINT, coreIntHandler);
 
 	/* Clean up */
 	pfree(env_postgis_gdal_enabled_drivers);
