@@ -1,19 +1,4 @@
-CREATE TEMPORARY TABLE _time AS SELECT now() t;
-
-CREATE FUNCTION _timecheck(label text, tolerated interval) RETURNS text
-AS $$
-DECLARE
-  ret TEXT;
-  lap INTERVAL;
-BEGIN
-  lap := now()-t FROM _time;
-  IF lap <= tolerated THEN ret := label || ' interrupted on time';
-  ELSE ret := label || ' interrupted late: ' || lap;
-  END IF;
-  UPDATE _time SET t = now();
-  RETURN ret;
-END;
-$$ LANGUAGE 'plpgsql' VOLATILE;
+set client_min_messages to WARNING;
 
 CREATE TEMP TABLE _inputs AS
 SELECT 1::int as id, ST_Collect(g) g FROM (
@@ -25,17 +10,18 @@ SELECT 1::int as id, ST_Collect(g) g FROM (
  ) foo
 ;
 
-UPDATE _time SET t = now(); -- reset time as creating tables spends some
+\i :regdir/utils/timecheck.sql
 
 -----------------
 -- ST_Buffer
 -----------------
 
 SET statement_timeout TO 100;
+
 select ST_Buffer(g,100) from _inputs WHERE id = 1;
 --( select (st_dumppoints(st_buffer(st_makepoint(0,0),10000,100000))).geom g) foo;
 -- it may take some more to interrupt st_buffer, see
-SELECT _timecheck('buffer', '200ms');
+SELECT _timecheck('buffer', '250ms');
 
 -- Not affected by old timeout
 SELECT '1', ST_NPoints(ST_Buffer('POINT(4 0)'::geometry, 2, 1));
